@@ -13,24 +13,23 @@
 // (ii) the Apache License v 2.0. (http://www.apache.org/licenses/LICENSE-2.0)
 //-----------------------------------------------------------------------------
 
-use std::boxed::Box;
 use std::fmt;
 use std::iter::FusedIterator;
 use std::marker::PhantomData;
 use std::rc::Rc;
 
 use crate::sql_type::FromSql;
+use crate::statement::Stmt;
 use crate::ColumnIndex;
 use crate::ColumnInfo;
-use crate::Connection;
-use crate::DpiConn;
 use crate::Result;
 use crate::SqlValue;
-use crate::Statement;
+
+#[allow(unused_imports)] // for links in doc comments
+use crate::Connection;
 
 pub struct RowSharedData {
     column_names: Vec<String>,
-    conn_handle: DpiConn,
 }
 
 /// Row in a result set of a select statement
@@ -40,14 +39,9 @@ pub struct Row {
 }
 
 impl Row {
-    pub(crate) fn new(
-        conn: &Connection,
-        column_names: Vec<String>,
-        column_values: Vec<SqlValue>,
-    ) -> Result<Row> {
+    pub(crate) fn new(column_names: Vec<String>, column_values: Vec<SqlValue>) -> Result<Row> {
         let shared = RowSharedData {
             column_names: column_names,
-            conn_handle: conn.handle.clone(),
         };
         Ok(Row {
             shared: Rc::new(shared),
@@ -107,13 +101,19 @@ impl fmt::Debug for Row {
     }
 }
 
+#[derive(Debug)]
+enum StmtHolder<'a> {
+    Borrowed(&'a Stmt),
+    Owned(Stmt),
+}
+
 /// Result set
+#[derive(Debug)]
 pub struct ResultSet<'a, T>
 where
     T: RowValue,
 {
-    stmt: Option<&'a Statement<'a>>,
-    pub(crate) stmt_boxed: Option<Box<Statement<'a>>>,
+    stmt: StmtHolder<'a>,
     phantom: PhantomData<T>,
 }
 
@@ -121,29 +121,24 @@ impl<'a, T> ResultSet<'a, T>
 where
     T: RowValue,
 {
-    pub(crate) fn new(stmt: &'a Statement<'a>) -> ResultSet<'a, T> {
+    pub(crate) fn new(stmt: &'a Stmt) -> ResultSet<'a, T> {
         ResultSet {
-            stmt: Some(stmt),
-            stmt_boxed: None,
+            stmt: StmtHolder::Borrowed(stmt),
             phantom: PhantomData,
         }
     }
 
-    pub(crate) fn from_conn(conn: &'a Connection, sql: &str) -> Result<ResultSet<'a, T>> {
-        Ok(ResultSet {
-            stmt: None,
-            stmt_boxed: Some(Box::new(conn.prepare(sql, &[])?)),
+    pub(crate) fn from_stmt(stmt: Stmt) -> ResultSet<'a, T> {
+        ResultSet {
+            stmt: StmtHolder::Owned(stmt),
             phantom: PhantomData,
-        })
+        }
     }
 
-    fn stmt(&self) -> &Statement {
-        if self.stmt.is_some() {
-            self.stmt.as_ref().unwrap()
-        } else if self.stmt_boxed.is_some() {
-            self.stmt_boxed.as_ref().unwrap().as_ref()
-        } else {
-            panic!("Both stmt and stmt_boxed are none!");
+    fn stmt(&self) -> &Stmt {
+        match &self.stmt {
+            &StmtHolder::Borrowed(stmt) => stmt,
+            &StmtHolder::Owned(ref stmt) => stmt,
         }
     }
 
@@ -167,29 +162,10 @@ where
 
 impl<'stmt, T> FusedIterator for ResultSet<'stmt, T> where T: RowValue {}
 
-impl<'stmt, T> fmt::Debug for ResultSet<'stmt, T>
-where
-    T: RowValue,
-{
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let stmt = self.stmt();
-        write!(
-            f,
-            "ResultSet {{ origin: {}, stmt: {:?} }}",
-            if self.stmt.is_some() {
-                "statement"
-            } else {
-                "connection"
-            },
-            stmt
-        )
-    }
-}
-
 /// A trait to get a row as specified type
 ///
-/// This is the return type of [`Connection.query_row_as`][Connection#method.query_row_as],
-/// [`Connection.query_row_as_named`][Connection#method.query_row_as_named] and [`Row.get_as`][Row#method.get_as].
+/// This is the return type of [`Connection::query_row_as`],
+/// [`Connection::query_row_as_named`] and [`Row::get_as`].
 ///
 /// The trait was added to fetch column values as a tuple.
 /// The oracle crate provides implementations for a type
@@ -253,7 +229,7 @@ impl RowValue for Row {
         let num_cols = row.column_values.len();
         let mut column_values = Vec::with_capacity(num_cols);
         for val in &row.column_values {
-            column_values.push(val.dup_by_handle(&row.shared.conn_handle)?);
+            column_values.push(val.dup_by_handle()?);
         }
         Ok(Row {
             shared: row.shared.clone(),
