@@ -18,6 +18,7 @@ use std::cell::RefCell;
 use std::fmt;
 use std::marker::PhantomData;
 use std::mem::MaybeUninit;
+use std::os::raw::c_char;
 use std::ptr;
 use std::rc::Rc;
 
@@ -470,7 +471,7 @@ impl<'conn> Statement<'conn> {
         let mut bind_names = Vec::with_capacity(bind_count);
         let mut bind_values = Vec::with_capacity(bind_count);
         if bind_count > 0 {
-            let mut names: Vec<*const i8> = vec![ptr::null_mut(); bind_count];
+            let mut names: Vec<*const c_char> = vec![ptr::null_mut(); bind_count];
             let mut lengths = vec![0; bind_count];
             chkerr!(
                 conn.ctxt(),
